<?php

namespace EventPortal\Http\Controllers\Admin;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\Render;
use EventPortal\Domain\Venue\Actions\DeleteVenue;
use EventPortal\Domain\Venue\Models\Venue;
use EventPortal\Enums\InertiaMethod;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Inertia\Response;

class VenueController extends Controller
{
    public function __construct()
    {
        $this->middleware('check-permission:venue.show')->only('show');
        $this->middleware('check-permission:venue.store')->only(['create', 'store']);
        $this->middleware('check-permission:venue.update')->only(['edit', 'update']);
        $this->middleware('check-permission:venue.destroy')->only('delete');
    }

    public function index(): Response
    {
        $table = Table::make('venue.index')
            ->query(Venue::query())
            ->request(request())
            ->get();

        $quickButton = Button::make('Create venue')
            ->method(InertiaMethod::VISIT())
            ->icon('plus')
            ->route('admin.venue.create');

        return Render::component('Venue/Index')
            ->key('venue.index')
            ->withTable($table)
            ->withQuickButton($quickButton)
            ->withTitle('Venues')
            ->get();
    }

    public function create(): Response
    {
        $form = Form::make('venue.create')->model(new Venue)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Venues', route('admin.venue.index'))
            ->add('Create an venue')
            ->get();

        return Render::component('Venue/Create')
            ->key('venue.create')
            ->withForm($form)
            ->withBreadcrumbs($breadcrumbs)
            ->get();
    }

    public function store(Request $request): RedirectResponse
    {
        Form::make('venue.create')
            ->model(new Venue)
            ->validate($request->all())
            ->save($request->all());

        toast()->success(trans('venue.created'));

        return redirect()->route('admin.venue.index');
    }

    public function edit(Venue $venue): Response
    {
        $form = Form::make('venue.edit')->model($venue)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Venues', route('admin.venue.index'))
            ->add($venue->name, route('admin.venue.show', $venue->id))
            ->add('Edit venue')
            ->get();

        return Render::component('Venue/Edit')
            ->key('venue.edit')
            ->withForm($form)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle($venue->name)
            ->get();
    }

    public function update(Request $request, Venue $venue): RedirectResponse
    {
        Form::make('venue.edit')
            ->model($venue)
            ->validate($request->all())
            ->save($request->all());

        toast()->success(trans('venue.updated'));

        return redirect()->route('admin.venue.index');
    }

    public function show(Venue $venue): Response
    {
        $form = Form::make('venue.show')->model($venue)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Venues', route('admin.venue.index'))
            ->add($venue->name)
            ->get();

        return Render::component('Venue/Show')
            ->key('venue.show')
            ->withData($form)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle($venue->name)
            ->get();
    }

    public function delete(Venue $venue, DeleteVenue $deleteVenue): RedirectResponse
    {
        $deleteVenue->execute($venue);

        toast()->success(trans('venue.deleted'));

        return redirect()->route('admin.venue.index');
    }
}
